@@ -9,7 +9,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001 - 2015 The SCons Foundation
+# Copyright (c) 2001 - 2017 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -31,7 +31,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "src/engine/SCons/Tool/midl.py rel_2.3.5:3347:d31d5a4e74b6 2015/07/31 14:36:10 bdbaddog"
+__revision__ = "src/engine/SCons/Tool/midl.py 74b2c53bc42290e911b334a6b44f187da698a668 2017/11/14 13:16:53 bdbaddog"
 
 import SCons.Action
 import SCons.Builder
@@ -39,26 +39,26 @@ import SCons.Defaults
 import SCons.Scanner.IDL
 import SCons.Util
 
-from MSCommon import msvc_exists
+from .MSCommon import msvc_exists
 
 def midl_emitter(target, source, env):
     """Produces a list of outputs from the MIDL compiler"""
-    base, ext = SCons.Util.splitext(str(target[0]))
+    base, _ = SCons.Util.splitext(str(target[0]))
     tlb = target[0]
     incl = base + '.h'
     interface = base + '_i.c'
-    t = [tlb, incl, interface]
+    targets = [tlb, incl, interface]
 
     midlcom = env['MIDLCOM']
 
     if midlcom.find('/proxy') != -1:
         proxy = base + '_p.c'
-        t.append(proxy)
+        targets.append(proxy)
     if midlcom.find('/dlldata') != -1:
         dlldata = base + '_data.c'
-        t.append(dlldata)
-    
-    return (t,source)
+        targets.append(dlldata)
+
+    return (targets, source)
 
 idl_scanner = SCons.Scanner.IDL.IDLScan()
 
